@@ -114,32 +114,7 @@
 `as_factor.declared` <- function(
     x, levels = c("default", "labels", "values", "both"), ordered = FALSE, ...
 ) {
-    levels <- match.arg(levels)
-    label <- attr(x, "label", exact = TRUE)
-    labels <- attr(x, "labels")
-
-    if (levels %in% c("default", "both")) {
-        if (levels == "both") {
-            names(labels) <- paste0("[", labels, "] ", names(labels))
-            attr(x, "labels") <- labels
-        }
-
-        vals <- sort(unique(x), na.last = TRUE)
-        x <- factor(to_labels(undeclare(x)), levels = to_labels(vals), ordered = ordered)
-
-    }
-    else if (levels == "labels") {
-        levs <- unname(labels)
-        labs <- names(labels)
-        x <- factor(to_labels(undeclare(x)), levels = sort(unique(labs)), ordered = ordered)
-    }
-    else if (levels == "values") {
-        levels <- unique(undeclare(sort(x, na.last = TRUE)))
-        
-        x <- factor(undeclare(x), levels, ordered = ordered)
-    }
-
-    structure(x, label = label)
+    as.factor(x, levels = levels, ordered = ordered, ... = ...)
 }
 
 `zap_labels.declared` <- function(x) {
